#ifndef DEVICEELEMENTWIDGET_H
#define DEVICEELEMENTWIDGET_H

#include "gmc-miscellaneousse_global.h"

#include <elementwidget.h>

#include "ruledeviceserviceexpressions.h"

class QValidator;
class QSortFilterProxyModel;

class TreeNode;
class TreeNodeModel;
class TreeNodeSortFilterProxyModel;
class ChoiceTreeNodeProxyModel;
class OneCheckTreeNodeModel;
class AuditComponentRootTreeNode;
class AuditTwoCheckTreeNodeModel;

namespace Ui
{
class DeviceElementWidget;
}

class GMCMISCELLANEOUSSESHARED_EXPORT DeviceElementWidget
: public ViewEditElementWidget
, public RuleDeviceExpressionsServiceClass
{

    Q_OBJECT

public:

    DeviceElementWidget( Module & /*module*/
                       , bool ( * /*isEditable*/ )( const Module & ), QWidget * /*parent*/ = nullptr );
    virtual ~DeviceElementWidget();

    void init();

    bool nameEditableEditMode() const;
    void setNameEditableEditMode( bool nameEditableEditMode );

    void beforeCreation() override;
    ElementData * exportElementData() const override;
    void afterSuccessfulRead( Element * /*testElement*/, ElementData * /*elementData*/ ) override;
    void afterSuccessfulUpdate() override;

    QString caption() const override;

protected:

    void reset() override;
    bool isContentValid() const override;
    void blockEditWidget( bool /*blocked*/ ) override;

    QString auditToString( qulonglong /*audit*/ ) const;

protected slots:

    void auditChanged();
    void importRecordData();
    void descriptionChanged();

protected:

    Ui::DeviceElementWidget *_ui;
    bool _nameEditableEditMode;
    int _descriptionMaxSize;
    QValidator *_nameValidator;

    TreeNode *_usersRootNode;
    TreeNodeModel *_usersTreeNodeModel;
    QSortFilterProxyModel *_usersProxyModel;

    TreeNode *_groupsRootNode;
    TreeNodeModel *_groupsTreeNodeModel;
    QSortFilterProxyModel *_groupsProxyModel;

    TreeNode *_categoriesRootNode;
    OneCheckTreeNodeModel *_categoriesTreeNodeModel;
    TreeNodeSortFilterProxyModel *_categoriesProxyModel;

    TreeNode *_levelsRootNode;
    ElementModel *_levelsElementModel;
    QSortFilterProxyModel *_levelsProxyModel;

    AuditTwoCheckTreeNodeModel *_auditTreeNodeModel;
    QSortFilterProxyModel *_auditProxyModel;

    TreeNode *_rulesRootNode;
    TreeNodeModel *_rulesTreeNodeModel;
    ChoiceTreeNodeProxyModel *_rulesChoiceModel;

};

#endif // DeviceElementWidget_H
