#ifndef MISCELLANEOUSELEMENTDATASE_H
#define MISCELLANEOUSELEMENTDATASE_H

#include "gmc-miscellaneousse_global.h"

#include <QtGlobal>
#include <QSet>
#include <QStringList>

#include <elementdata.h>

class Record;
class RuleRecord;

//******************************************************************************

class GMCMISCELLANEOUSSESHARED_EXPORT MacElementData : public ElementData
{

public:

    MacElementData();
    MacElementData( const MacElementData & /*elementData*/ );
    virtual ~MacElementData();

    const MacElementData & operator=( const MacElementData & /*elementData*/ );

    int value() const;
    void setValue( int /*value*/);

    QString synonym() const;
    void setSynonym( const QString &synonym );

    QStringList users() const;
    void setUsers( const QStringList &users );

protected:

    int _value;
    QString _synonym;
    QStringList _users;

};

//******************************************************************************

class GMCMISCELLANEOUSSESHARED_EXPORT AuditElementData : public ElementData
{

public:

    AuditElementData();
    AuditElementData( const AuditElementData & /*elementData*/ );
    virtual ~AuditElementData();

    const AuditElementData & operator=( const AuditElementData & /*elementData*/ );

    bool isValid() const;
    void setValid(bool isValid);

    qulonglong successAudit() const;
    void setSuccessAudit(const qulonglong &successAudit);

    qulonglong failAudit() const;
    void setFailAudit(const qulonglong &failAudit);

protected:

    bool _isValid;
    qulonglong _successAudit;
    qulonglong _failAudit;

};

//******************************************************************************

class GMCMISCELLANEOUSSESHARED_EXPORT CapabilitiesElementData : public ElementData
{

public:

    CapabilitiesElementData();
    CapabilitiesElementData( const CapabilitiesElementData & /*elementData*/ );
    virtual ~CapabilitiesElementData();

    const CapabilitiesElementData & operator=( const CapabilitiesElementData & /*elementData*/ );

    bool isValid() const;
    void setValid(bool isValid);

    qulonglong linuxCapabilities() const;
    void setLinuxCapabilities(const qulonglong &linuxCapabilities);

    qulonglong parsecCapabilities() const;
    void setParsecCapabilities(const qulonglong &parsecCapabilities);

protected:

    bool _isValid;
    qulonglong _linuxCapabilities;
    qulonglong _parsecCapabilities;

};

//******************************************************************************

class GMCMISCELLANEOUSSESHARED_EXPORT RuleLightElementData : public ElementData
{

public:

    RuleLightElementData();
    RuleLightElementData( const RuleLightElementData & /*elementData*/ );
    virtual ~RuleLightElementData();

    const RuleLightElementData & operator=( const RuleLightElementData & /*elementData*/ );

    bool isEnabled() const;
    void setEnabled( bool /*isEnabled*/ );

protected:

    bool _isEnabled;

};

//******************************************************************************

class GMCMISCELLANEOUSSESHARED_EXPORT DeviceLightElementData : virtual public RuleLightElementData
{

public:

    DeviceLightElementData();
    DeviceLightElementData( const DeviceLightElementData & /*elementData*/ );
    virtual ~DeviceLightElementData();

    const DeviceLightElementData & operator=( const DeviceLightElementData & /*elementData*/ );

    const QString &user() const;
    void setUser( const QString & /*user*/ );

    const QString &group() const;
    void setGroup( const QString & /*group*/ );

    int mode() const;
    void setMode(int /*mode*/ );

protected:

    QString _user;
    QString _group;
    int _mode;

};

//******************************************************************************

class GMCMISCELLANEOUSSESHARED_EXPORT RuleHeavyElementData : public virtual RuleLightElementData
{

public:

    RuleHeavyElementData();
    RuleHeavyElementData( const RuleHeavyElementData & /*elementData*/ );
    virtual ~RuleHeavyElementData();

    const RuleHeavyElementData & operator=( const RuleHeavyElementData & /*elementData*/ );

    const QString &description() const;
    void setDescription(const QString &description);

   QList< Record *> &expressions();

protected:

    QString _description;
    QList< Record * > _expressions;

};

//******************************************************************************

class GMCMISCELLANEOUSSESHARED_EXPORT DeviceHeavyElementData
: public DeviceLightElementData, public RuleHeavyElementData
{

public:

    DeviceHeavyElementData();
    DeviceHeavyElementData( const DeviceHeavyElementData & /*elementData*/ );
    virtual ~DeviceHeavyElementData();

    const DeviceHeavyElementData & operator=( const DeviceHeavyElementData & /*elementData*/ );

    int level() const;
    void setLevel(int level);

    qulonglong categories() const;
    void setCategories(const qulonglong &categories);

    qulonglong successAudit() const;
    void setSuccessAudit(const qulonglong &successAudit);

    qulonglong failAudit() const;
    void setFailAudit(const qulonglong &failAudit);

    QStringList &rules();

protected:

    int _level;
    qulonglong _categories;
    qulonglong _successAudit;
    qulonglong _failAudit;

    QStringList _rules;

};

//******************************************************************************

#endif
